#include <LEDA/system/stream.h>
#include <LEDA/graphics/graphwin.h>
#include <LEDA/graph/graph.h>
#include <LEDA/graph/edge_array.h>
#include <LEDA/graph/node_array.h>

using namespace leda;
using std::ifstream;
using std::cin;
using std::cout;
using std::endl;

#define SIZE 57 //n×n行列の時n+1に設定する
#define LOW 10000.0

void input(double A[SIZE][SIZE]);//データ入力関数
void search_min(double A[SIZE][SIZE], int *di, int *dj, double *min ,int cnt);//最小値のペアを探す
void copy_matorix(double A[SIZE][SIZE],double B[SIZE][SIZE]);//行列Aを行列Bにコピー
void init_matorix(double A[SIZE][SIZE]);//行列の初期化
void create_matorix(double M[SIZE][SIZE],double Temp[SIZE][SIZE],int di,int dj,int cnt);//表の更新
array<node> update_node_array(array<node> V,int di, int dj, int index);//グラフとノードのindexの同期
void edit_graph(array<node> NAME, array<string> F, node ROOT,int check, int index);//グラフの設定関数

GRAPH<string,double> G;

int main()
{

  	double Temp[SIZE][SIZE]={},M[SIZE][SIZE]={};
	double min;
	int i,cnt,di=0,dj=0;

	array<node> V(2*SIZE);
	array<node> NAME(2*SIZE);
	array<string> F(SIZE-1);
	
	input(M);

  	ifstream fin("filename");
	for(i=0; i < SIZE-1; i++){
		fin>>F[i];
	}
	for(i=0; i < SIZE-1; i++){
		V[i]=G.new_node();
		NAME[i]=V[i];
	}
	
	int index = i;
	int check = index;
	
	for(cnt=SIZE-1; cnt > 1; cnt--){
		search_min(M,&di,&dj,&min,cnt);
		V[cnt]=G.new_node();
		NAME[index]=V[cnt];
		index++;
		G.new_edge(V[di],V[cnt],min/2);
		G.new_edge(V[dj],V[cnt],min/2);
		V = update_node_array(V,di,dj,cnt);
		init_matorix(Temp);
		create_matorix(M,Temp,di,dj,cnt);
	}
	
	node ROOT = G.new_node();
	G.new_edge(V[0],ROOT);

	edit_graph(NAME,F,ROOT,check,index);
	
	return 0;
}

void input(double A[SIZE][SIZE]) //データ入力関数
{
	int i,j;
	
	ifstream fin("fileout");

	for(i=0;i<SIZE;i++){
		for(j=0;j<i+1;j++){
			if(i==j){
				A[i][j]=0;
			}else{
				fin>>A[i][j];
			}
    		}
    	}
}

void search_min(double A[SIZE][SIZE], int *di, int *dj, double *min, int cnt) //最小値のペアを探す
{
	int i,j;
	*min = LOW;
	for(i=1; i < cnt; i++){
		for(j=0; j < i+1; j++){
			if(A[i][j] != 0 && A[i][j] < *min){
				 *min = A[i][j];
				 *di = i;
				 *dj = j;
			}
		}
	}
}

void copy_matorix(double A[SIZE][SIZE],double B[SIZE][SIZE]) //行列Aを行列Bにコピー
{
	int i,j;
	
	for(i=0; i < SIZE; i++){
		for(j=0; j < SIZE; j++){
			B[i][j] = A[i][j];
		}
	}
}

void init_matorix(double A[SIZE][SIZE]) //行列の初期化
{
	int i,j;

	for(i=0; i < SIZE; i++){
		for(j=0; j < SIZE; j++){
			A[i][j] = 0;
		}
	}
	
}

void create_matorix(double M[SIZE][SIZE],double Temp[SIZE][SIZE],int di,int dj,int cnt) //表の更新
{
	int i,j,flag;
	int temp_cnt=cnt-1;
	int temp_i=1,temp_j=0;
	double val=0;
	
	//表の縮小
	for(i=1; i < cnt; i++){
		if(i==di || i==dj){//di、dj行をスキップ
			continue;
		}
		for(j=0, temp_j=0, flag=0; j < i; j++){
			if(j==di || j==dj){//di、dj列をスキップ
				continue;
			}
			Temp[temp_i][temp_j]=M[i][j];
			temp_j++;
			flag = 1;		
		}
		if(flag == 1) {
			temp_i++;
		}
	}
	
        //距離表の更新
	for(i=0,j=0; i < cnt; i++){
		if(i==di || i==dj){//di、dj行をスキップ
			continue;
		}
		
		val = M[i][di]+M[i][dj]+M[di][i]+M[dj][i];
		if(val == 0){
			Temp[temp_cnt-1][j] = 0;
		} else {
			Temp[temp_cnt-1][j] = val/2;
		}
		j++;
	}	
	copy_matorix(Temp,M);

}

array<node> update_node_array(array<node> V,int di, int dj, int cnt)
{
	int i,j;
	array<node> TMP(2*SIZE);
	
	for(i=0,j=0; i < cnt; i++){
		if(i != di && i != dj ){
			TMP[j] = V[i];
			j++;
		}
	}

	TMP[j] = V[cnt];

	return TMP;
}
 
void edit_graph(array<node> NAME, array<string> F, node ROOT,int check, int index)//グラフの設定関数
{
	int i;
	
	//グラフの設定(詳細)
	GraphWin gw(G,900,900,"UPGMA");

	gw.set_edge_direction(undirected_edge);
	gw.set_edge_color(black);
	
	//元のノードの設定
	for(i=0;i < check; i++){
		gw.set_label(NAME[i],F[i]);
		gw.set_color(NAME[i],red);
		gw.set_shape(NAME[i],rectangle_node);
		gw.set_width(NAME[i],50);
		gw.set_height(NAME[i],25);
	}

	//合成したノードの設定
	for(i=check; i < index; i++){
		gw.set_label(NAME[i]," ");
		gw.set_color(NAME[i],green);
		gw.set_width(NAME[i],10);
   		gw.set_height(NAME[i],10);
	}

	//ROOTの設定
	gw.set_label(ROOT,"ROOT");
	gw.set_color(ROOT,yellow);
	gw.set_shape(ROOT,ellipse_node);
	gw.set_width(ROOT,50);
	gw.set_height(ROOT,25);
	point p(250,450);
	gw.set_position(ROOT,p);
	
	
	gw.open();
}
